@@ -53,9 +53,18 @@ config_load_from_file (Configuration *config, const gchar *path, GList **message
     groups = g_key_file_get_groups (key_file, NULL);
     for (i = 0; groups[i]; i++)
     {
-        gchar **keys;
+        gchar **keys, *group;
         int j;
 
+        /* Move keys from deprecated [SeatDefaults] into [Seat:*] */
+        group = groups[i];
+        if (strcmp (group, "SeatDefaults") == 0)
+        {
+            if (messages)
+                *messages = g_list_append (*messages, g_strdup ("  [SeatDefaults] is now called [Seat:*], please update this configuration"));
+            group = "Seat:*";
+        }
+
         keys = g_key_file_get_keys (key_file, groups[i], NULL, error);
         if (!keys)
             break;
@@ -64,16 +73,16 @@ config_load_from_file (Configuration *config, const gchar *path, GList **message
         {
             gchar *value, *k;
 
-            if (messages && g_str_has_prefix (groups[i], "Seat:") && strcmp (keys[j], "xdg-seat") == 0)
-                *messages = g_list_append (*messages, g_strdup_printf ("  [%s] contains deprecated option xdg-seat, this can be safely removed", groups[i]));
-            if (messages && strcmp (groups[i], "LightDM") == 0 && strcmp (keys[j], "logind-load-seats") == 0)
+            if (messages && g_str_has_prefix (group, "Seat:") && strcmp (keys[j], "xdg-seat") == 0)
+                *messages = g_list_append (*messages, g_strdup_printf ("  [%s] contains deprecated option xdg-seat, this can be safely removed", group));
+            if (messages && strcmp (group, "LightDM") == 0 && strcmp (keys[j], "logind-load-seats") == 0)
                 *messages = g_list_append (*messages, g_strdup ("  [LightDM] contains deprecated option logind-load-seats, this can be safely removed"));
 
             value = g_key_file_get_value (key_file, groups[i], keys[j], NULL);
-            g_key_file_set_value (config->priv->key_file, groups[i], keys[j], value);
+            g_key_file_set_value (config->priv->key_file, group, keys[j], value);
             g_free (value);
 
-            k = g_strdup_printf ("%s]%s", groups[i], keys[j]);
+            k = g_strdup_printf ("%s]%s", group, keys[j]);
             g_hash_table_insert (config->priv->key_sources, k, source_path);
         }
 
@@ -337,9 +346,7 @@ config_init (Configuration *config)
 static void
 config_finalize (GObject *object)
 {
-    Configuration *self;
-
-    self = CONFIGURATION (object);
+    Configuration *self = CONFIGURATION (object);
 
     g_free (self->priv->dir);
     g_key_file_free (self->priv->key_file);
