@@ -12,6 +12,8 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <gio/gio.h>
+#include <gio/gunixsocketaddress.h>
 #include <security/pam_appl.h>
 
 #include "lightdm/greeter.h"
@@ -50,11 +52,15 @@ typedef struct
     /* TRUE if the daemon can reuse this greeter */
     gboolean resettable;
 
+    /* Socket connection to daemon */
+    GSocket *socket;
+
     /* Channel to write to daemon */
     GIOChannel *to_server_channel;
 
     /* Channel to read from daemon */
     GIOChannel *from_server_channel;
+    guint from_server_watch;
 
     /* Data read from the daemon */
     guint8 *read_buffer;
@@ -141,6 +147,43 @@ static void request_iface_init (GAsyncResultIface *iface);
 #define REQUEST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), request_get_type (), Request))
 G_DEFINE_TYPE_WITH_CODE (Request, request, G_TYPE_OBJECT, G_IMPLEMENT_INTERFACE (G_TYPE_ASYNC_RESULT, request_iface_init));
 
+static gboolean from_server_cb (GIOChannel *source, GIOCondition condition, gpointer data);
+
+GType
+lightdm_prompt_type_get_type (void)
+{
+    static GType enum_type = 0;
+  
+    if (G_UNLIKELY(enum_type == 0)) {
+        static const GEnumValue values[] = {
+            { LIGHTDM_PROMPT_TYPE_QUESTION, "LIGHTDM_PROMPT_TYPE_QUESTION", "question" },
+            { LIGHTDM_PROMPT_TYPE_SECRET, "LIGHTDM_PROMPT_TYPE_SECRET", "secret" },
+            { 0, NULL, NULL }
+        };
+        enum_type = g_enum_register_static (g_intern_static_string ("LightDMPromptType"), values);
+    }
+
+    return enum_type;
+}
+
+GType
+lightdm_message_type_get_type (void)
+{
+    static GType enum_type = 0;
+  
+    if (G_UNLIKELY(enum_type == 0)) {
+        static const GEnumValue values[] = {
+            { LIGHTDM_MESSAGE_TYPE_INFO, "LIGHTDM_MESSAGE_TYPE_INFO", "info" },
+            { LIGHTDM_MESSAGE_TYPE_ERROR, "LIGHTDM_MESSAGE_TYPE_ERROR", "error" },
+            { 0, NULL, NULL }
+        };
+        enum_type = g_enum_register_static (g_intern_static_string ("LightDMMessageType"), values);
+    }
+
+    return enum_type;
+}
+
+
 /**
  * lightdm_greeter_new:
  *
@@ -311,6 +354,66 @@ get_message_length (guint8 *message, gsize message_length)
     return read_int (message, message_length, &offset);
 }
 
+static gboolean
+connect_to_daemon (LightDMGreeter *greeter)
+{
+    LightDMGreeterPrivate *priv = GET_PRIVATE (greeter);
+    const gchar *to_server_fd, *from_server_fd, *pipe_path;
+    GError *error = NULL;
+
+    if (priv->to_server_channel || priv->from_server_channel)
+        return TRUE;
+
+    /* Use private connection if one exists */  
+    to_server_fd = g_getenv ("LIGHTDM_TO_SERVER_FD");
+    from_server_fd = g_getenv ("LIGHTDM_FROM_SERVER_FD");
+    pipe_path = g_getenv ("LIGHTDM_GREETER_PIPE");
+    if (to_server_fd && from_server_fd)
+    {
+        priv->to_server_channel = g_io_channel_unix_new (atoi (to_server_fd));
+        priv->from_server_channel = g_io_channel_unix_new (atoi (from_server_fd));
+    }
+    else if (pipe_path)
+    {
+        GSocketAddress *address;
+        gboolean result;
+
+        priv->socket = g_socket_new (G_SOCKET_FAMILY_UNIX, G_SOCKET_TYPE_STREAM, G_SOCKET_PROTOCOL_DEFAULT, &error);
+        if (!priv->socket)
+            return FALSE;
+
+        address = g_unix_socket_address_new (pipe_path);
+        result = g_socket_connect (priv->socket, address, NULL, &error);
+        g_object_unref (address);
+        if (!result)
+        {
+            g_warning ("Failed to connect to greeter socket %s: %s", pipe_path, error->message);
+            g_clear_error (&error);
+            return FALSE;
+        }
+
+        priv->from_server_channel = g_io_channel_unix_new (g_socket_get_fd (priv->socket));
+        priv->to_server_channel = g_io_channel_ref (priv->from_server_channel);
+    }
+    else
+    {
+        g_warning ("Unable to determine socket to daemon");
+        return FALSE;
+    }
+
+    priv->from_server_watch = g_io_add_watch (priv->from_server_channel, G_IO_IN, from_server_cb, greeter);
+
+    if (!g_io_channel_set_encoding (priv->to_server_channel, NULL, &error) ||
+        !g_io_channel_set_encoding (priv->from_server_channel, NULL, &error))
+    {
+        g_warning ("Failed to set encoding on from server channel to binary: %s", error->message);
+        g_clear_error (&error);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 static gboolean
 send_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
 {
@@ -320,7 +423,7 @@ send_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
     GError *error = NULL;
     guint32 stated_length;
 
-    if (!priv->to_server_channel)
+    if (!connect_to_daemon (greeter))
         return FALSE;
 
     /* Double check that we're sending well-formed messages.  If we say we're
@@ -346,6 +449,8 @@ send_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
         if (error)
             g_warning ("Error writing to daemon: %s", error->message);
         g_clear_error (&error);
+        if (status == G_IO_STATUS_AGAIN)
+            continue;
         if (status != G_IO_STATUS_NORMAL)
             return FALSE;
         data_length -= n_written;
@@ -625,8 +730,8 @@ recv_message (LightDMGreeter *greeter, gsize *length, gboolean block)
     guint8 *buffer;
     GError *error = NULL;
 
-    if (!priv->from_server_channel)
-        return NULL;
+    if (!connect_to_daemon (greeter))
+        return FALSE;
 
     /* Read the header, or the whole message if we already have that */
     n_to_read = HEADER_SIZE;
@@ -644,7 +749,12 @@ recv_message (LightDMGreeter *greeter, gsize *length, gboolean block)
         if (error)
             g_warning ("Error reading from server: %s", error->message);
         g_clear_error (&error);
-        if (status != G_IO_STATUS_NORMAL)
+        if (status == G_IO_STATUS_AGAIN)
+        {
+            if (block)
+                continue;
+        }
+        else if (status != G_IO_STATUS_NORMAL)
             break;
 
         g_debug ("Read %zi bytes from daemon", n_read);
@@ -767,6 +877,7 @@ lightdm_greeter_connect_to_daemon (LightDMGreeter *greeter, GCancellable *cancel
 
 /**
  * lightdm_greeter_connect_to_daemon_finish:
+ * @greeter: The greeter the the request was done with
  * @result: A #GAsyncResult.
  * @error: return location for a #GError, or %NULL
  *
@@ -795,6 +906,7 @@ lightdm_greeter_connect_to_daemon_sync (LightDMGreeter *greeter, GError **error)
 {
     LightDMGreeterPrivate *priv;
     Request *request;
+    gboolean result;
 
     g_return_val_if_fail (LIGHTDM_IS_GREETER (greeter), FALSE);
 
@@ -816,9 +928,10 @@ lightdm_greeter_connect_to_daemon_sync (LightDMGreeter *greeter, GError **error)
         g_free (message);
     } while (!request->complete);
 
+    result = request->complete;
     g_object_unref (request);
 
-    return request->complete;
+    return result;
 }
 
 /**
@@ -845,7 +958,7 @@ lightdm_greeter_connect_sync (LightDMGreeter *greeter, GError **error)
  *
  * Get a hint.
  *
- * Return value: The value for this hint or #NULL if not set.
+ * Return value: (nullable): The value for this hint or #NULL if not set.
  **/
 const gchar *
 lightdm_greeter_get_hint (LightDMGreeter *greeter, const gchar *name)
@@ -980,7 +1093,7 @@ lightdm_greeter_get_has_guest_account_hint (LightDMGreeter *greeter)
  *
  * Get the user to select by default.
  *
- * Return value: A username
+ * Return value: (nullable): A username or %NULL if no particular user should be selected.
  */
 const gchar *
 lightdm_greeter_get_select_user_hint (LightDMGreeter *greeter)
@@ -1012,9 +1125,9 @@ lightdm_greeter_get_select_guest_hint (LightDMGreeter *greeter)
  * lightdm_greeter_get_autologin_user_hint:
  * @greeter: A #LightDMGreeter
  *
- * Get the user account to automatically logg into when the timer expires.
+ * Get the user account to automatically log into when the timer expires.
  *
- * Return value: The user account to automatically log into.
+ * Return value: (nullable): The user account to automatically log into or %NULL if none configured.
  */
 const gchar *
 lightdm_greeter_get_autologin_user_hint (LightDMGreeter *greeter)
@@ -1319,7 +1432,7 @@ lightdm_greeter_get_is_authenticated (LightDMGreeter *greeter)
  *
  * Get the user that is being authenticated.
  *
- * Return value: The username of the authentication user being authenticated or #NULL if no authentication in progress.
+ * Return value: (nullable): The username of the authentication user being authenticated or #NULL if no authentication in progress.
  */
 const gchar *
 lightdm_greeter_get_authentication_user (LightDMGreeter *greeter)
@@ -1484,6 +1597,7 @@ lightdm_greeter_ensure_shared_data_dir (LightDMGreeter *greeter, const gchar *us
  * @result: A #GAsyncResult.
  * @greeter: A #LightDMGreeter
  *
+ * Function to call from lightdm_greeter_ensure_shared_data_dir callback.
  *
  * Return value: The path to the shared directory, free with g_free.
  **/
@@ -1551,39 +1665,9 @@ static void
 lightdm_greeter_init (LightDMGreeter *greeter)
 {
     LightDMGreeterPrivate *priv = GET_PRIVATE (greeter);
-    const gchar *fd;
 
     priv->read_buffer = g_malloc (HEADER_SIZE);
     priv->hints = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-
-    fd = g_getenv ("LIGHTDM_TO_SERVER_FD");
-    if (fd)
-    {
-        GError *error = NULL;
-
-        priv->to_server_channel = g_io_channel_unix_new (atoi (fd));
-        g_io_channel_set_encoding (priv->to_server_channel, NULL, &error);
-        if (error)
-            g_warning ("Failed to set encoding on to server channel to binary: %s\n", error->message);
-        g_clear_error (&error);
-    }
-    else
-        g_warning ("No LIGHTDM_TO_SERVER_FD environment variable");
-
-    fd = g_getenv ("LIGHTDM_FROM_SERVER_FD");
-    if (fd)
-    {
-        GError *error = NULL;
-
-        priv->from_server_channel = g_io_channel_unix_new (atoi (fd));
-        g_io_channel_set_encoding (priv->from_server_channel, NULL, &error);
-        if (error)
-            g_warning ("Failed to set encoding on from server channel to binary: %s\n", error->message);
-        g_clear_error (&error);
-        g_io_add_watch (priv->from_server_channel, G_IO_IN, from_server_cb, greeter);
-    }
-    else
-        g_warning ("No LIGHTDM_FROM_SERVER_FD environment variable");
 }
 
 static void
@@ -1660,12 +1744,26 @@ lightdm_greeter_finalize (GObject *object)
     LightDMGreeter *self = LIGHTDM_GREETER (object);
     LightDMGreeterPrivate *priv = GET_PRIVATE (self);
 
+    g_clear_object (&priv->socket);
     if (priv->to_server_channel)
         g_io_channel_unref (priv->to_server_channel);
     if (priv->from_server_channel)
         g_io_channel_unref (priv->from_server_channel);
-    g_free (priv->authentication_user);
+    if (priv->from_server_watch)
+        g_source_remove (priv->from_server_watch);
+    priv->from_server_watch = 0;
+    g_clear_pointer (&priv->read_buffer, g_free);
+    g_list_free_full (priv->responses_received, g_free);
+    priv->responses_received = NULL;
+    g_list_free_full (priv->connect_requests, g_object_unref);
+    priv->connect_requests = NULL;
+    g_list_free_full (priv->start_session_requests, g_object_unref);
+    priv->start_session_requests = NULL;
+    g_list_free_full (priv->ensure_shared_data_dir_requests, g_object_unref);
+    priv->ensure_shared_data_dir_requests = NULL;
+    g_clear_pointer (&priv->authentication_user, g_free);
     g_hash_table_unref (priv->hints);
+    priv->hints = NULL;
 
     G_OBJECT_CLASS (lightdm_greeter_parent_class)->finalize (object);
 }
@@ -1687,7 +1785,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                                                           "default-session-hint",
                                                           "Default session hint",
                                                           NULL,
-                                                          G_PARAM_READWRITE));
+                                                          G_PARAM_READABLE));
 
     g_object_class_install_property (object_class,
                                      PROP_HIDE_USERS_HINT,
@@ -1811,7 +1909,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                       G_STRUCT_OFFSET (LightDMGreeterClass, show_prompt),
                       NULL, NULL,
                       NULL,
-                      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_INT);
+                      G_TYPE_NONE, 2, G_TYPE_STRING, lightdm_prompt_type_get_type ());
 
     /**
      * LightDMGreeter::show-message:
@@ -1829,7 +1927,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
                       G_STRUCT_OFFSET (LightDMGreeterClass, show_message),
                       NULL, NULL,
                       NULL,
-                      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_INT);
+                      G_TYPE_NONE, 2, G_TYPE_STRING, lightdm_message_type_get_type ());
 
     /**
      * LightDMGreeter::authentication-complete:
@@ -1916,8 +2014,7 @@ request_finalize (GObject *object)
     Request *request = REQUEST (object);
 
     g_free (request->dir);
-    if (request->cancellable)
-        g_object_unref (request->cancellable);
+    g_clear_object (&request->cancellable);
 
     G_OBJECT_CLASS (request_parent_class)->finalize (object);
 }
