@@ -23,7 +23,7 @@ namespace QLightDM
 class Q_DECL_EXPORT Greeter : public QObject
 {
     Q_OBJECT
-    
+
     Q_PROPERTY(bool authenticated READ isAuthenticated ) //NOTFIY authenticationComplete
     Q_PROPERTY(QString authenticationUser READ authenticationUser )
     Q_PROPERTY(QString defaultSession READ defaultSessionHint CONSTANT)
@@ -33,7 +33,7 @@ class Q_DECL_EXPORT Greeter : public QObject
     Q_PROPERTY(QString hostname READ hostname CONSTANT)
     Q_PROPERTY(bool hasGuestAccount READ hasGuestAccountHint CONSTANT)
     Q_PROPERTY(bool locked READ lockHint CONSTANT)
-    
+
     Q_PROPERTY(QString hostname READ hostname CONSTANT)
 
     Q_ENUMS(PromptType MessageType)
@@ -52,7 +52,7 @@ public:
     explicit Greeter(QObject* parent=0);
     virtual ~Greeter();
 
-    QString getHint(const QString &name) const;   
+    QString getHint(const QString &name) const;
     QString defaultSessionHint() const;
     bool hideUsersHint() const;
     bool showManualLoginHint() const;
@@ -71,6 +71,7 @@ public:
     QString hostname() const;
 
 public Q_SLOTS:
+    bool connectToDaemonSync();
     bool connectSync();
     void authenticate(const QString &username=QString());
     void authenticateAsGuest();
@@ -78,7 +79,9 @@ public Q_SLOTS:
     void authenticateRemote(const QString &session=QString(), const QString &username=QString());
     void respond(const QString &response);
     void cancelAuthentication();
+    void cancelAutologin();
     void setLanguage (const QString &language);
+    void setResettable (bool resettable);
     bool startSessionSync(const QString &session=QString());
     QString ensureSharedDataDirSync(const QString &username);
 
@@ -87,6 +90,8 @@ Q_SIGNALS:
     void showPrompt(QString text, QLightDM::Greeter::PromptType type);
     void authenticationComplete();
     void autologinTimerExpired();
+    void idle();
+    void reset();
 
 private:
     GreeterPrivate *d_ptr;
