@@ -28,12 +28,14 @@ public:
     LightDMGreeter *ldmGreeter;
 protected:
     Greeter* q_ptr;
-    
+
     static void cb_showPrompt(LightDMGreeter *greeter, const gchar *text, LightDMPromptType type, gpointer data);
     static void cb_showMessage(LightDMGreeter *greeter, const gchar *text, LightDMMessageType type, gpointer data);
     static void cb_authenticationComplete(LightDMGreeter *greeter, gpointer data);
     static void cb_autoLoginExpired(LightDMGreeter *greeter, gpointer data);
-    
+    static void cb_idle(LightDMGreeter *greeter, gpointer data);
+    static void cb_reset(LightDMGreeter *greeter, gpointer data);
+
 private:
     Q_DECLARE_PUBLIC(Greeter)
 };
@@ -46,19 +48,21 @@ GreeterPrivate::GreeterPrivate(Greeter *parent) :
 #endif
     ldmGreeter = lightdm_greeter_new();
 
-    g_signal_connect (ldmGreeter, "show-prompt", G_CALLBACK (cb_showPrompt), this);
-    g_signal_connect (ldmGreeter, "show-message", G_CALLBACK (cb_showMessage), this);
-    g_signal_connect (ldmGreeter, "authentication-complete", G_CALLBACK (cb_authenticationComplete), this);
-    g_signal_connect (ldmGreeter, "autologin-timer-expired", G_CALLBACK (cb_autoLoginExpired), this);
+    g_signal_connect (ldmGreeter, LIGHTDM_GREETER_SIGNAL_SHOW_PROMPT, G_CALLBACK (cb_showPrompt), this);
+    g_signal_connect (ldmGreeter, LIGHTDM_GREETER_SIGNAL_SHOW_MESSAGE, G_CALLBACK (cb_showMessage), this);
+    g_signal_connect (ldmGreeter, LIGHTDM_GREETER_SIGNAL_AUTHENTICATION_COMPLETE, G_CALLBACK (cb_authenticationComplete), this);
+    g_signal_connect (ldmGreeter, LIGHTDM_GREETER_SIGNAL_AUTOLOGIN_TIMER_EXPIRED, G_CALLBACK (cb_autoLoginExpired), this);
+    g_signal_connect (ldmGreeter, LIGHTDM_GREETER_SIGNAL_IDLE, G_CALLBACK (cb_idle), this);
+    g_signal_connect (ldmGreeter, LIGHTDM_GREETER_SIGNAL_RESET, G_CALLBACK (cb_reset), this);
 }
 
 void GreeterPrivate::cb_showPrompt(LightDMGreeter *greeter, const gchar *text, LightDMPromptType type, gpointer data)
 {
     Q_UNUSED(greeter);
-    
+
     GreeterPrivate *that = static_cast<GreeterPrivate*>(data);
     QString message = QString::fromUtf8(text);
-    
+
     Q_EMIT that->q_func()->showPrompt(message, type == LIGHTDM_PROMPT_TYPE_QUESTION ?
                                                Greeter::PromptTypeQuestion : Greeter::PromptTypeSecret);
 }
@@ -88,6 +92,20 @@ void GreeterPrivate::cb_autoLoginExpired(LightDMGreeter *greeter, gpointer data)
     Q_EMIT that->q_func()->autologinTimerExpired();
 }
 
+void GreeterPrivate::cb_idle(LightDMGreeter *greeter, gpointer data)
+{
+    Q_UNUSED(greeter);
+    GreeterPrivate *that = static_cast<GreeterPrivate*>(data);
+    Q_EMIT that->q_func()->idle();
+}
+
+void GreeterPrivate::cb_reset(LightDMGreeter *greeter, gpointer data)
+{
+    Q_UNUSED(greeter);
+    GreeterPrivate *that = static_cast<GreeterPrivate*>(data);
+    Q_EMIT that->q_func()->reset();
+}
+
 Greeter::Greeter(QObject *parent) :
     QObject(parent),
     d_ptr(new GreeterPrivate(this))
@@ -100,10 +118,16 @@ Greeter::~Greeter()
 }
 
 
+bool Greeter::connectToDaemonSync()
+{
+    Q_D(Greeter);
+    return lightdm_greeter_connect_to_daemon_sync(d->ldmGreeter, NULL);
+}
+
 bool Greeter::connectSync()
 {
     Q_D(Greeter);
-    return lightdm_greeter_connect_sync(d->ldmGreeter, NULL);
+    return lightdm_greeter_connect_to_daemon_sync(d->ldmGreeter, NULL);
 }
 
 void Greeter::authenticate(const QString &username)
@@ -142,6 +166,12 @@ void Greeter::cancelAuthentication()
     lightdm_greeter_cancel_authentication(d->ldmGreeter);
 }
 
+void Greeter::cancelAutologin()
+{
+    Q_D(Greeter);
+    lightdm_greeter_cancel_autologin(d->ldmGreeter);
+}
+
 bool Greeter::inAuthentication() const
 {
     Q_D(const Greeter);
@@ -166,6 +196,12 @@ void Greeter::setLanguage (const QString &language)
     lightdm_greeter_set_language(d->ldmGreeter, language.toLocal8Bit().constData());
 }
 
+void Greeter::setResettable (bool resettable)
+{
+    Q_D(Greeter);
+    lightdm_greeter_set_resettable(d->ldmGreeter, resettable);
+}
+
 bool Greeter::startSessionSync(const QString &session)
 {
     Q_D(Greeter);
