@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -10,8 +10,8 @@
  * license.
  */
 
-#ifndef _USER_H_
-#define _USER_H_
+#ifndef USER_H_
+#define USER_H_
 
 #include <glib-object.h>
 #include <gio/gio.h>
@@ -36,8 +36,6 @@ typedef struct
 
 User *accounts_get_user_by_name (const gchar *username);
 
-User *accounts_get_user_by_uid (uid_t uid);
-
 User *accounts_get_current_user (void);
 
 GType user_get_type (void);
@@ -48,8 +46,6 @@ uid_t user_get_uid (User *user);
 
 gid_t user_get_gid (User *user);
 
-const gchar *user_get_gecos (User *user);
-
 const gchar *user_get_home_directory (User *user);
 
 const gchar *user_get_shell (User *user);
@@ -58,10 +54,10 @@ const gchar *user_get_xsession (User *user);
 
 void user_set_xsession (User *user, const gchar *session);
 
-const gchar *user_get_locale (User *user);
+const gchar *user_get_language (User *user);
 
 void user_set_language (User *user, const gchar *language);
 
 G_END_DECLS
 
-#endif /* _USER_H_ */
+#endif /* USER_H_ */
