@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -23,6 +23,10 @@ G_BEGIN_DECLS
 #define PROCESS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PROCESS_TYPE, Process))
 #define PROCESS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PROCESS_TYPE, ProcessClass))
 
+#define PROCESS_SIGNAL_GOT_DATA   "got-data"
+#define PROCESS_SIGNAL_GOT_SIGNAL "got-signal"
+#define PROCESS_SIGNAL_STOPPED    "stopped"
+
 typedef struct ProcessPrivate ProcessPrivate;
 
 typedef struct
