@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -9,8 +9,8 @@
  * license.
  */
 
-#ifndef _SEAT_XREMOTE_H_
-#define _SEAT_XREMOTE_H_
+#ifndef SEAT_XREMOTE_H_
+#define SEAT_XREMOTE_H_
 
 #include <glib-object.h>
 #include "seat.h"
@@ -34,4 +34,4 @@ GType seat_xremote_get_type (void);
 
 G_END_DECLS
 
-#endif /* _SEAT_XREMOTE_H_ */
+#endif /* SEAT_XREMOTE_H_ */
