@@ -27,8 +27,10 @@
 #include "login1.h"
 #include "guest-account.h"
 #include "shared-data-manager.h"
+#include "greeter-socket.h"
 
 enum {
+    CREATE_GREETER,
     GOT_MESSAGES,
     AUTHENTICATION_COMPLETE,
     STOPPED,
@@ -94,14 +96,17 @@ struct SessionPrivate
     XAuthority *x_authority;
     gboolean x_authority_use_system_location;
 
+    /* Socket to allow greeters to connect to (if allowed) */
+    GreeterSocket *greeter_socket;
+
     /* Remote host this session is being controlled from */
     gchar *remote_host_name;
 
     /* Console kit cookie */
     gchar *console_kit_cookie;
 
-    /* login1 session */
-    gchar *login1_session;
+    /* login1 session ID */
+    gchar *login1_session_id;
 
     /* Environment to set in child */
     GList *env;
@@ -136,8 +141,7 @@ session_set_config (Session *session, SessionConfig *config)
 {
     g_return_if_fail (session != NULL);
 
-    if (session->priv->config)
-        g_object_unref (session->priv->config);
+    g_clear_object (&session->priv->config);
     session->priv->config = g_object_ref (config);
 }
 
@@ -152,7 +156,7 @@ const gchar *
 session_get_session_type (Session *session)
 {
     g_return_val_if_fail (session != NULL, NULL);
-    return session_config_get_session_type (session_get_config (session));
+    return session_config_get_session_type (session->priv->config);
 }
 
 void
@@ -213,6 +217,10 @@ session_set_display_server (Session *session, DisplayServer *display_server)
 {
     g_return_if_fail (session != NULL);
     g_return_if_fail (display_server != NULL);
+
+    if (session->priv->display_server == display_server)
+        return;
+
     if (session->priv->display_server)
     {
         display_server_disconnect_session (session->priv->display_server, session);
@@ -248,11 +256,7 @@ void
 session_set_x_authority (Session *session, XAuthority *authority, gboolean use_system_location)
 {
     g_return_if_fail (session != NULL);
-    if (session->priv->x_authority)
-    {
-        g_object_unref (session->priv->x_authority);
-        session->priv->x_authority = NULL;
-    }
+    g_clear_object (&session->priv->x_authority);
     if (authority)
         session->priv->x_authority = g_object_ref (authority);
     session->priv->x_authority_use_system_location = use_system_location;
@@ -312,7 +316,7 @@ session_get_env (Session *session, const gchar *name)
     link = find_env_entry (session, name);
     if (!link)
         return NULL;
-  
+
     entry = link->data;
 
     return entry + strlen (name) + 1;
@@ -324,7 +328,7 @@ session_unset_env (Session *session, const gchar *name)
     GList *link;
 
     g_return_if_fail (session != NULL);
-  
+
     link = find_env_entry (session, name);
     if (!link)
         return;
@@ -486,9 +490,7 @@ from_child_cb (GIOChannel *source, GIOCondition condition, gpointer data)
     {
         g_free (session->priv->username);
         session->priv->username = username;
-        if (session->priv->user)
-            g_object_unref (session->priv->user);
-        session->priv->user = NULL;
+        g_clear_object (&session->priv->user);
     }
     else
         g_free (username);
@@ -554,6 +556,14 @@ session_get_is_started (Session *session)
     return session->priv->pid != 0;
 }
 
+static Greeter *
+create_greeter_cb (GreeterSocket *socket, Session *session)
+{
+    Greeter *greeter;
+    g_signal_emit (session, signals[CREATE_GREETER], 0, &greeter);
+    return greeter;
+}
+
 static gboolean
 session_real_start (Session *session)
 {
@@ -654,6 +664,13 @@ session_get_username (Session *session)
     return session->priv->username;
 }
 
+const gchar *
+session_get_login1_session_id (Session *session)
+{
+    g_return_val_if_fail (session != NULL, NULL);
+    return session->priv->login1_session_id;
+}
+
 const gchar *
 session_get_console_kit_cookie (Session *session)
 {
@@ -731,10 +748,18 @@ session_get_authentication_result_string (Session *session)
 void
 session_run (Session *session)
 {
+    g_return_if_fail (session != NULL);
     g_return_if_fail (session->priv->display_server != NULL);
     return SESSION_GET_CLASS (session)->run (session);
 }
 
+gboolean
+session_get_is_run (Session *session)
+{
+    g_return_val_if_fail (session != NULL, FALSE);
+    return session->priv->command_run;
+}
+
 static void
 session_real_run (Session *session)
 {
@@ -790,6 +815,38 @@ session_real_run (Session *session)
         }
     }
 
+    /* Open socket to allow in-session greeter */
+    if (session->priv->config && session_config_get_allow_greeter (session->priv->config))
+    {
+        gchar *run_dir, *dir, *path;
+        GError *error = NULL;
+
+        run_dir = config_get_string (config_get_instance (), "LightDM", "run-directory");
+        dir = g_build_filename (run_dir, session->priv->username, NULL);
+        g_free (run_dir);
+
+        if (g_mkdir_with_parents (dir, S_IRWXU) < 0)
+            l_warning (session, "Failed to create greeter socket dir %s: %s", dir, strerror (errno));
+        if (getuid () == 0)
+        {
+            if (chown (dir, user_get_uid (session_get_user (session)), user_get_gid (session_get_user (session))) < 0)
+                l_warning (session, "Failed to set ownership of greeter socket dir: %s", strerror (errno));
+        }
+
+        path = g_build_filename (dir, "greeter-socket", NULL);
+        session->priv->greeter_socket = greeter_socket_new (path);
+        g_signal_connect (session->priv->greeter_socket, GREETER_SOCKET_SIGNAL_CREATE_GREETER, G_CALLBACK (create_greeter_cb), session);
+        session_set_env (session, "LIGHTDM_GREETER_PIPE", path);
+        g_free (path);
+        g_free (dir);
+
+        if (!greeter_socket_start (session->priv->greeter_socket, &error))
+        {
+            l_warning (session, "Failed to start greeter socket: %s\n", error->message);
+            g_clear_error (&error);
+        }
+    }
+
     if (session->priv->log_filename)
         l_debug (session, "Logging to %s", session->priv->log_filename);
     write_string (session, session->priv->log_filename);
@@ -808,7 +865,7 @@ session_real_run (Session *session)
     for (i = 0; i < argc; i++)
         write_string (session, session->priv->argv[i]);
 
-    session->priv->login1_session = read_string_from_child (session);
+    session->priv->login1_session_id = read_string_from_child (session);
     session->priv->console_kit_cookie = read_string_from_child (session);
 }
 
@@ -818,8 +875,8 @@ session_lock (Session *session)
     g_return_if_fail (session != NULL);
     if (getuid () == 0)
     {
-        if (session->priv->login1_session)
-            login1_service_lock_session (login1_service_get_instance (), session->priv->login1_session);
+        if (session->priv->login1_session_id)
+            login1_service_lock_session (login1_service_get_instance (), session->priv->login1_session_id);
         else if (session->priv->console_kit_cookie)
             ck_lock_session (session->priv->console_kit_cookie);
     }
@@ -831,8 +888,8 @@ session_unlock (Session *session)
     g_return_if_fail (session != NULL);
     if (getuid () == 0)
     {
-        if (session->priv->login1_session)
-            login1_service_unlock_session (login1_service_get_instance (), session->priv->login1_session);
+        if (session->priv->login1_session_id)
+            login1_service_unlock_session (login1_service_get_instance (), session->priv->login1_session_id);
         else if (session->priv->console_kit_cookie)
             ck_unlock_session (session->priv->console_kit_cookie);
     }
@@ -844,8 +901,8 @@ session_activate (Session *session)
     g_return_if_fail (session != NULL);
     if (getuid () == 0)
     {
-        if (session->priv->login1_session)
-            login1_service_activate_session (login1_service_get_instance (), session->priv->login1_session);
+        if (session->priv->login1_session_id)
+            login1_service_activate_session (login1_service_get_instance (), session->priv->login1_session_id);
         else if (session->priv->console_kit_cookie)
             ck_activate_session (session->priv->console_kit_cookie);
     }
@@ -919,10 +976,8 @@ session_finalize (GObject *object)
     Session *self = SESSION (object);
     int i;
 
-    if (self->priv->config)
-        g_object_unref (self->priv->config);
-    if (self->priv->display_server)
-        g_object_unref (self->priv->display_server);
+    g_clear_object (&self->priv->config);
+    g_clear_object (&self->priv->display_server);
     if (self->priv->pid)
         kill (self->priv->pid, SIGKILL);
     close (self->priv->to_child_input);
@@ -934,8 +989,7 @@ session_finalize (GObject *object)
     if (self->priv->child_watch)
         g_source_remove (self->priv->child_watch);
     g_free (self->priv->username);
-    if (self->priv->user)
-        g_object_unref (self->priv->user);
+    g_clear_object (&self->priv->user);
     g_free (self->priv->pam_service);
     for (i = 0; i < self->priv->messages_length; i++)
         g_free ((char *) self->priv->messages[i].msg);
@@ -944,10 +998,9 @@ session_finalize (GObject *object)
     g_free (self->priv->log_filename);
     g_free (self->priv->tty);
     g_free (self->priv->xdisplay);
-    if (self->priv->x_authority)
-        g_object_unref (self->priv->x_authority);
+    g_clear_object (&self->priv->x_authority);
     g_free (self->priv->remote_host_name);
-    g_free (self->priv->login1_session);
+    g_free (self->priv->login1_session_id);
     g_free (self->priv->console_kit_cookie);
     g_list_free_full (self->priv->env, g_free);
     g_strfreev (self->priv->argv);
@@ -967,8 +1020,18 @@ session_class_init (SessionClass *klass)
 
     g_type_class_add_private (klass, sizeof (SessionPrivate));
 
+    signals[CREATE_GREETER] =
+        g_signal_new (SESSION_SIGNAL_CREATE_GREETER,
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (SessionClass, create_greeter),
+                      g_signal_accumulator_first_wins,
+                      NULL,
+                      NULL,
+                      GREETER_TYPE, 0);
+
     signals[GOT_MESSAGES] =
-        g_signal_new ("got-messages",
+        g_signal_new (SESSION_SIGNAL_GOT_MESSAGES,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SessionClass, got_messages),
@@ -977,7 +1040,7 @@ session_class_init (SessionClass *klass)
                       G_TYPE_NONE, 0);
 
     signals[AUTHENTICATION_COMPLETE] =
-        g_signal_new ("authentication-complete",
+        g_signal_new (SESSION_SIGNAL_AUTHENTICATION_COMPLETE,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SessionClass, authentication_complete),
@@ -986,7 +1049,7 @@ session_class_init (SessionClass *klass)
                       G_TYPE_NONE, 0);
 
     signals[STOPPED] =
-        g_signal_new ("stopped",
+        g_signal_new (SESSION_SIGNAL_STOPPED,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SessionClass, stopped),
