@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2013 Canonical Ltd.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -21,7 +21,7 @@
 #include "unity-system-compositor.h"
 #include "configuration.h"
 #include "process.h"
-#include "greeter.h"
+#include "greeter-session.h"
 #include "vt.h"
 
 struct UnitySystemCompositorPrivate
@@ -39,9 +39,6 @@ struct UnitySystemCompositorPrivate
     gint vt;
     gboolean have_vt_ref;
 
-    /* TRUE if should show hardware cursor */
-    gboolean enable_hardware_cursor;
-
     /* Pipes to communicate with compositor */
     int to_compositor_pipe[2];
     int from_compositor_pipe[2];
@@ -61,6 +58,10 @@ struct UnitySystemCompositorPrivate
 
     /* TRUE when received ready signal */
     gboolean is_ready;
+
+    /* Counters for Mir IDs to use */
+    int next_session_id;
+    int next_greeter_id;  
 };
 
 G_DEFINE_TYPE (UnitySystemCompositor, unity_system_compositor, DISPLAY_SERVER_TYPE);
@@ -85,6 +86,8 @@ void
 unity_system_compositor_set_command (UnitySystemCompositor *compositor, const gchar *command)
 {
     g_return_if_fail (compositor != NULL);
+    g_return_if_fail (command != NULL);
+
     g_free (compositor->priv->command);
     compositor->priv->command = g_strdup (command);
 }
@@ -120,13 +123,6 @@ unity_system_compositor_set_vt (UnitySystemCompositor *compositor, gint vt)
     }
 }
 
-void
-unity_system_compositor_set_enable_hardware_cursor (UnitySystemCompositor *compositor, gboolean enable_cursor)
-{
-    g_return_if_fail (compositor != NULL);
-    compositor->priv->enable_hardware_cursor = enable_cursor;
-}
-
 void
 unity_system_compositor_set_timeout (UnitySystemCompositor *compositor, gint timeout)
 {
@@ -145,7 +141,8 @@ write_message (UnitySystemCompositor *compositor, guint16 id, const guint8 *payl
     data[1] = id & 0xFF;
     data[2] = payload_length >> 8;
     data[3] = payload_length & 0xFF;
-    memcpy (data + 4, payload, payload_length);
+    if (payload)
+        memcpy (data + 4, payload, payload_length);
 
     errno = 0;
     if (write (compositor->priv->to_compositor_pipe[1], data, data_length) != data_length)
@@ -179,15 +176,28 @@ static void
 unity_system_compositor_connect_session (DisplayServer *display_server, Session *session)
 {
     UnitySystemCompositor *compositor = UNITY_SYSTEM_COMPOSITOR (display_server);
-    const gchar *name;
+
+    session_set_env (session, "XDG_SESSION_TYPE", "mir");
 
     if (compositor->priv->socket)
-        session_set_env (session, "MIR_SOCKET", compositor->priv->socket);
-    if (IS_GREETER (session))
-        name = "greeter-0";
-    else
-        name = "session-0";
-    session_set_env (session, "MIR_SERVER_NAME", name);
+        session_set_env (session, "MIR_SERVER_HOST_SOCKET", compositor->priv->socket);
+
+    if (!session_get_env (session, "MIR_SERVER_NAME"))
+    {
+        gchar *name;
+        if (IS_GREETER_SESSION (session))
+        {
+            name = g_strdup_printf ("greeter-%d", compositor->priv->next_greeter_id);
+            compositor->priv->next_greeter_id++;
+        }
+        else
+        {
+            name = g_strdup_printf ("session-%d", compositor->priv->next_session_id);
+            compositor->priv->next_session_id++;
+        }
+        session_set_env (session, "MIR_SERVER_NAME", name);
+        g_free (name);
+    }
 
     if (compositor->priv->vt >= 0)
     {
@@ -200,7 +210,8 @@ unity_system_compositor_connect_session (DisplayServer *display_server, Session
 static void
 unity_system_compositor_disconnect_session (DisplayServer *display_server, Session *session)
 {
-    session_unset_env (session, "MIR_SOCKET");
+    session_unset_env (session, "XDG_SESSION_TYPE");
+    session_unset_env (session, "MIR_SERVER_HOST_SOCKET");
     session_unset_env (session, "MIR_SERVER_NAME");
     session_unset_env (session, "XDG_VTNR");
 }
@@ -235,7 +246,7 @@ read_cb (GIOChannel *source, GIOCondition condition, gpointer data)
     gsize n_to_read = 0;
     guint16 id, payload_length;
     /*guint8 *payload;*/
-  
+
     if (condition == G_IO_HUP)
     {
         l_debug (compositor, "Compositor closed communication channel");
@@ -430,20 +441,18 @@ unity_system_compositor_start (DisplayServer *server)
     g_string_append_printf (command, " --from-dm-fd %d --to-dm-fd %d", compositor->priv->to_compositor_pipe[0], compositor->priv->from_compositor_pipe[1]);
     if (compositor->priv->vt > 0)
         g_string_append_printf (command, " --vt %d", compositor->priv->vt);
-    if (compositor->priv->enable_hardware_cursor)
-        g_string_append (command, " --enable-hardware-cursor=true");
     process_set_command (compositor->priv->process, command->str);
     g_string_free (command, TRUE);
 
     /* Start the compositor */
-    g_signal_connect (compositor->priv->process, "stopped", G_CALLBACK (stopped_cb), compositor);
+    g_signal_connect (compositor->priv->process, PROCESS_SIGNAL_STOPPED, G_CALLBACK (stopped_cb), compositor);
     result = process_start (compositor->priv->process, FALSE);
 
     /* Close compostor ends of the pipes */
     close (compositor->priv->to_compositor_pipe[0]);
-    compositor->priv->to_compositor_pipe[0] = 0;
+    compositor->priv->to_compositor_pipe[0] = -1;
     close (compositor->priv->from_compositor_pipe[1]);
-    compositor->priv->from_compositor_pipe[1] = 0;
+    compositor->priv->from_compositor_pipe[1] = -1;
 
     if (!result)
         return FALSE;
@@ -457,7 +466,7 @@ unity_system_compositor_start (DisplayServer *server)
 
     return TRUE;
 }
- 
+
 static void
 unity_system_compositor_stop (DisplayServer *server)
 {
@@ -470,16 +479,18 @@ unity_system_compositor_init (UnitySystemCompositor *compositor)
     compositor->priv = G_TYPE_INSTANCE_GET_PRIVATE (compositor, UNITY_SYSTEM_COMPOSITOR_TYPE, UnitySystemCompositorPrivate);
     compositor->priv->vt = -1;
     compositor->priv->command = g_strdup ("unity-system-compositor");
-    compositor->priv->socket = g_strdup ("/tmp/mir_socket");
+    compositor->priv->socket = g_strdup ("/run/mir_socket");
     compositor->priv->timeout = -1;
+    compositor->priv->to_compositor_pipe[0] = -1;
+    compositor->priv->to_compositor_pipe[1] = -1;
+    compositor->priv->from_compositor_pipe[0] = -1;
+    compositor->priv->from_compositor_pipe[1] = -1;
 }
 
 static void
 unity_system_compositor_finalize (GObject *object)
 {
-    UnitySystemCompositor *self;
-
-    self = UNITY_SYSTEM_COMPOSITOR (object);  
+    UnitySystemCompositor *self = UNITY_SYSTEM_COMPOSITOR (object);
 
     if (self->priv->process)
     {
@@ -495,7 +506,7 @@ unity_system_compositor_finalize (GObject *object)
     close (self->priv->from_compositor_pipe[0]);
     close (self->priv->from_compositor_pipe[1]);
     g_io_channel_unref (self->priv->from_compositor_channel);
-    if (self->priv->from_compositor_watch)      
+    if (self->priv->from_compositor_watch)
         g_source_remove (self->priv->from_compositor_watch);
     g_free (self->priv->read_buffer);
     if (self->priv->timeout_source)
