@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2010-2011 Robert Ancell.
  * Author: Robert Ancell <robert.ancell@canonical.com>
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
  * Foundation, either version 3 of the License, or (at your option) any later
@@ -101,7 +101,7 @@ open_tcp_socket (GSocketFamily family, guint port, const gchar *listen_address,
 {
     GSocket *socket;
     GSocketAddress *address;
-  
+
     socket = g_socket_new (family, G_SOCKET_TYPE_STREAM, G_SOCKET_PROTOCOL_TCP, error);
     if (!socket)
         return NULL;
@@ -143,7 +143,7 @@ vnc_server_start (VNCServer *server)
     if (error)
         g_warning ("Failed to create IPv4 VNC socket: %s", error->message);
     g_clear_error (&error);
-  
+
     if (server->priv->socket)
     {
         source = g_socket_create_source (server->priv->socket, G_IO_IN, NULL);
@@ -179,17 +179,13 @@ vnc_server_init (VNCServer *server)
 static void
 vnc_server_finalize (GObject *object)
 {
-    VNCServer *self;
-
-    self = VNC_SERVER (object);
+    VNCServer *self = VNC_SERVER (object);
 
     g_free (self->priv->listen_address);
-    if (self->priv->socket)
-        g_object_unref (self->priv->socket);
-    if (self->priv->socket6)
-        g_object_unref (self->priv->socket6);
-  
-    G_OBJECT_CLASS (vnc_server_parent_class)->finalize (object);  
+    g_clear_object (&self->priv->socket);
+    g_clear_object (&self->priv->socket6);
+
+    G_OBJECT_CLASS (vnc_server_parent_class)->finalize (object);
 }
 
 static void
@@ -197,12 +193,12 @@ vnc_server_class_init (VNCServerClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-    object_class->finalize = vnc_server_finalize;  
+    object_class->finalize = vnc_server_finalize;
 
     g_type_class_add_private (klass, sizeof (VNCServerPrivate));
 
     signals[NEW_CONNECTION] =
-        g_signal_new ("new-connection",
+        g_signal_new (VNC_SERVER_SIGNAL_NEW_CONNECTION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (VNCServerClass, new_connection),
