@@ -13,11 +13,14 @@
 #define X_SERVER_LOCAL_H_
 
 #include "x-server.h"
+#include "process.h"
 
 G_BEGIN_DECLS
 
 #define X_SERVER_LOCAL_TYPE    (x_server_local_get_type())
 #define X_SERVER_LOCAL(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), X_SERVER_LOCAL_TYPE, XServerLocal))
+#define X_SERVER_LOCAL_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST ((klass), X_SERVER_LOCAL_TYPE, XServerLocalClass))
+#define X_SERVER_LOCAL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), X_SERVER_LOCAL_TYPE, XServerLocalClass))
 #define IS_X_SERVER_LOCAL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), X_SERVER_LOCAL_TYPE))
 
 typedef struct XServerLocalPrivate XServerLocalPrivate;
@@ -31,6 +34,10 @@ typedef struct
 typedef struct
 {
     XServerClass parent_class;
+    ProcessRunFunc (*get_run_function)(XServerLocal *server);
+    gboolean (*get_log_stdout)(XServerLocal *server);  
+    void (*add_args)(XServerLocal *server, GString *command);
+    gboolean (*start)(DisplayServer *server);
 } XServerLocalClass;
 
 const gchar *x_server_local_get_version (void);
@@ -55,8 +62,6 @@ void x_server_local_set_layout (XServerLocal *server, const gchar *layout);
 
 void x_server_local_set_xdg_seat (XServerLocal *server, const gchar *xdg_seat);
 
-void x_server_local_set_sharevts (XServerLocal *server, gboolean sharevts);
-
 void x_server_local_set_allow_tcp (XServerLocal *server, gboolean allow_tcp);
 
 void x_server_local_set_xdmcp_server (XServerLocal *server, const gchar *hostname);
@@ -71,12 +76,6 @@ void x_server_local_set_xdmcp_key (XServerLocal *server, const gchar *key);
 
 void x_server_local_set_background (XServerLocal *server, const gchar *background);
 
-void x_server_local_set_mir_id (XServerLocal *server, const gchar *id);
-
-const gchar *x_server_local_get_mir_id (XServerLocal *server);
-
-void x_server_local_set_mir_socket (XServerLocal *server, const gchar *socket);
-
 const gchar *x_server_local_get_authority_file_path (XServerLocal *server);
 
 G_END_DECLS
