@@ -11,6 +11,7 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <netinet/in.h>
 #include <pwd.h>
 #include <unistd.h>
@@ -36,7 +37,7 @@
 
 #define LOGIN_PROMPT "login:"
 
-static int console_fd = -1;
+static int tty_fd = -1;
 
 static GList *user_entries = NULL;
 static GList *getpwent_link = NULL;
@@ -72,6 +73,13 @@ struct pam_handle
     struct pam_conv conversation;
 };
 
+int
+gethostname (char *name, size_t len)
+{
+   snprintf (name, len, "lightdm-test");
+   return 0;
+}
+
 uid_t
 getuid (void)
 {
@@ -234,14 +242,14 @@ open_wrapper (const char *func, const char *pathname, int flags, mode_t mode)
 
     _open = (int (*)(const char *pathname, int flags, mode_t mode)) dlsym (RTLD_NEXT, func);
 
-    if (strcmp (pathname, "/dev/console") == 0)
+    if (strcmp (pathname, "/dev/tty0") == 0)
     {
-        if (console_fd < 0)
+        if (tty_fd < 0)
         {
-            console_fd = _open ("/dev/null", flags, mode);
-            fcntl (console_fd, F_SETFD, FD_CLOEXEC);
+            tty_fd = _open ("/dev/null", flags, mode);
+            fcntl (tty_fd, F_SETFD, FD_CLOEXEC);
         }
-        return console_fd;
+        return tty_fd;
     }
 
     new_path = redirect_path (pathname);
@@ -350,12 +358,13 @@ access (const char *pathname, int mode)
     gchar *new_path = NULL;
     int ret;
 
-    /* Look like systemd is always running */
-    if (strcmp (pathname, "/run/systemd/seats/") == 0)
-        return 1;
-
     _access = (int (*)(const char *pathname, int mode)) dlsym (RTLD_NEXT, "access");
 
+    if (strcmp (pathname, "/dev/tty0") == 0)
+        return F_OK;
+    if (strcmp (pathname, "/sys/class/tty/tty0/active") == 0)
+        return F_OK;
+
     new_path = redirect_path (pathname);
     ret = _access (new_path, mode);
     g_free (new_path);
@@ -520,9 +529,9 @@ ioctl (int d, unsigned long request, ...)
     int (*_ioctl) (int d, int request, ...);
 
     _ioctl = (int (*)(int d, int request, ...)) dlsym (RTLD_NEXT, "ioctl");
-    if (d > 0 && d == console_fd)
+    if (d > 0 && d == tty_fd)
     {
-        struct vt_stat *console_state;
+        struct vt_stat *vt_state;
         int vt;
         va_list ap;
 
@@ -530,9 +539,9 @@ ioctl (int d, unsigned long request, ...)
         {
         case VT_GETSTATE:
             va_start (ap, request);
-            console_state = va_arg (ap, struct vt_stat *);
+            vt_state = va_arg (ap, struct vt_stat *);
             va_end (ap);
-            console_state->v_active = active_vt;
+            vt_state->v_active = active_vt;
             break;
         case VT_ACTIVATE:
             va_start (ap, request);
@@ -608,35 +617,48 @@ int
 bind (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
 {
     int port = 0, redirected_port = 0;
+    const char *path;
     int (*_bind) (int sockfd, const struct sockaddr *addr, socklen_t addrlen);
     const struct sockaddr *modified_addr = addr;
-    struct sockaddr_in temp_addr;
-    struct sockaddr_in6 temp_addr6;
+    struct sockaddr_in temp_addr_in;
+    struct sockaddr_in6 temp_addr_in6;
+    struct sockaddr_un temp_addr_un;
     int retval;
 
     _bind = (int (*)(int sockfd, const struct sockaddr *addr, socklen_t addrlen)) dlsym (RTLD_NEXT, "bind");
 
     switch (addr->sa_family)
     {
+    case AF_UNIX:
+        path = ((const struct sockaddr_un *) addr)->sun_path;
+        if (path[0] != '\0')
+        {
+            gchar *new_path = redirect_path (path);
+            memcpy (&temp_addr_un, addr, sizeof (struct sockaddr_un));
+            strncpy (temp_addr_un.sun_path, new_path, sizeof (temp_addr_un.sun_path) - 1);
+            g_free (new_path);
+            modified_addr = (struct sockaddr *) &temp_addr_un;
+        }
+        break;
     case AF_INET:
         port = ntohs (((const struct sockaddr_in *) addr)->sin_port);
         redirected_port = find_port_redirect (port);
-        memcpy (&temp_addr, addr, sizeof (struct sockaddr_in));
-        modified_addr = (struct sockaddr *) &temp_addr;
+        memcpy (&temp_addr_in, addr, sizeof (struct sockaddr_in));
+        modified_addr = (struct sockaddr *) &temp_addr_in;
         if (redirected_port != 0)
-            temp_addr.sin_port = htons (redirected_port);
+            temp_addr_in.sin_port = htons (redirected_port);
         else
-            temp_addr.sin_port = 0;
+            temp_addr_in.sin_port = 0;
         break;
     case AF_INET6:
         port = ntohs (((const struct sockaddr_in6 *) addr)->sin6_port);
         redirected_port = find_port_redirect (port);
-        memcpy (&temp_addr6, addr, sizeof (struct sockaddr_in6));
-        modified_addr = (struct sockaddr *) &temp_addr6;
+        memcpy (&temp_addr_in6, addr, sizeof (struct sockaddr_in6));
+        modified_addr = (struct sockaddr *) &temp_addr_in6;
         if (redirected_port != 0)
-            temp_addr6.sin6_port = htons (redirected_port);
+            temp_addr_in6.sin6_port = htons (redirected_port);
         else
-            temp_addr6.sin6_port = 0;
+            temp_addr_in6.sin6_port = 0;
         break;
     }
 
@@ -646,20 +668,20 @@ bind (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
     switch (addr->sa_family)
     {
     case AF_INET:
-        temp_addr_len = sizeof (temp_addr);
-        getsockname (sockfd, &temp_addr, &temp_addr_len);
+        temp_addr_len = sizeof (temp_addr_in);
+        getsockname (sockfd, &temp_addr_in, &temp_addr_len);
         if (redirected_port == 0)
         {
-            redirected_port = ntohs (temp_addr.sin_port);
+            redirected_port = ntohs (temp_addr_in.sin_port);
             add_port_redirect (port, redirected_port);
         }
         break;
     case AF_INET6:
-        temp_addr_len = sizeof (temp_addr6);
-        getsockname (sockfd, &temp_addr6, &temp_addr_len);
+        temp_addr_len = sizeof (temp_addr_in6);
+        getsockname (sockfd, &temp_addr_in6, &temp_addr_len);
         if (redirected_port == 0)
         {
-            redirected_port = ntohs (temp_addr6.sin6_port);
+            redirected_port = ntohs (temp_addr_in6.sin6_port);
             add_port_redirect (port, redirected_port);
         }
         break;
@@ -668,27 +690,42 @@ bind (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
     return retval;
 }
 
+#include <ctype.h>
+
 int
 connect (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
 {
     int port, redirected_port;
+    const char *path;
     const struct sockaddr *modified_addr = addr;
-    struct sockaddr_in temp_addr;
-    struct sockaddr_in6 temp_addr6;
+    struct sockaddr_in temp_addr_in;
+    struct sockaddr_in6 temp_addr_in6;
+    struct sockaddr_un temp_addr_un;
     int (*_connect) (int sockfd, const struct sockaddr *addr, socklen_t addrlen);
 
     _connect = (int (*)(int sockfd, const struct sockaddr *addr, socklen_t addrlen)) dlsym (RTLD_NEXT, "connect");
 
     switch (addr->sa_family)
     {
+    case AF_UNIX:
+        path = ((const struct sockaddr_un *) addr)->sun_path;
+        if (path[0] != '\0') 
+        {
+            gchar *new_path = redirect_path (path);
+            memcpy (&temp_addr_un, addr, addrlen);
+            strncpy (temp_addr_un.sun_path, new_path, sizeof (temp_addr_un.sun_path) - 1);
+            g_free (new_path);
+            modified_addr = (struct sockaddr *) &temp_addr_un;
+        }
+        break;
     case AF_INET:
         port = ntohs (((const struct sockaddr_in *) addr)->sin_port);
         redirected_port = find_port_redirect (port);
         if (redirected_port != 0) 
         {
-            memcpy (&temp_addr, addr, sizeof (struct sockaddr_in));
-            temp_addr.sin_port = htons (redirected_port);
-            modified_addr = (struct sockaddr *) &temp_addr;
+            memcpy (&temp_addr_in, addr, sizeof (struct sockaddr_in));
+            temp_addr_in.sin_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr_in;
         }
         break;
     case AF_INET6:
@@ -696,9 +733,9 @@ connect (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
         redirected_port = find_port_redirect (port);
         if (redirected_port != 0) 
         {
-            memcpy (&temp_addr6, addr, sizeof (struct sockaddr_in6));
-            temp_addr6.sin6_port = htons (redirected_port);
-            modified_addr = (struct sockaddr *) &temp_addr6;
+            memcpy (&temp_addr_in6, addr, sizeof (struct sockaddr_in6));
+            temp_addr_in6.sin6_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr_in6;
         }
         break;
     }
@@ -710,23 +747,36 @@ ssize_t
 sendto (int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen)
 {
     int port, redirected_port;
+    const char *path;
     const struct sockaddr *modified_addr = dest_addr;
-    struct sockaddr_in temp_addr;
-    struct sockaddr_in6 temp_addr6;
+    struct sockaddr_in temp_addr_in;
+    struct sockaddr_in6 temp_addr_in6;
+    struct sockaddr_un temp_addr_un;  
     ssize_t (*_sendto) (int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen);
 
     _sendto = (ssize_t (*)(int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen)) dlsym (RTLD_NEXT, "sendto");
 
     switch (dest_addr->sa_family)
     {
+    case AF_UNIX:
+        path = ((const struct sockaddr_un *) dest_addr)->sun_path;
+        if (path[0] != '\0') 
+        {
+            gchar *new_path = redirect_path (path);
+            memcpy (&temp_addr_un, dest_addr, sizeof (struct sockaddr_un));
+            strncpy (temp_addr_un.sun_path, new_path, sizeof (temp_addr_un.sun_path) - 1);
+            g_free (new_path);
+            modified_addr = (struct sockaddr *) &temp_addr_un;
+        }
+        break;
     case AF_INET:
         port = ntohs (((const struct sockaddr_in *) dest_addr)->sin_port);
         redirected_port = find_port_redirect (port);
         if (redirected_port != 0) 
         {
-            memcpy (&temp_addr, dest_addr, sizeof (struct sockaddr_in));
-            temp_addr.sin_port = htons (redirected_port);
-            modified_addr = (struct sockaddr *) &temp_addr;
+            memcpy (&temp_addr_in, dest_addr, sizeof (struct sockaddr_in));
+            temp_addr_in.sin_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr_in;
         }
         break;
     case AF_INET6:
@@ -734,9 +784,9 @@ sendto (int sockfd, const void *buf, size_t len, int flags, const struct sockadd
         redirected_port = find_port_redirect (port);
         if (redirected_port != 0) 
         {
-            memcpy (&temp_addr6, dest_addr, sizeof (struct sockaddr_in6));
-            temp_addr6.sin6_port = htons (redirected_port);
-            modified_addr = (struct sockaddr *) &temp_addr6;
+            memcpy (&temp_addr_in6, dest_addr, sizeof (struct sockaddr_in6));
+            temp_addr_in6.sin6_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr_in6;
         }
         break;
     }
@@ -749,7 +799,7 @@ close (int fd)
 {
     int (*_close) (int fd);
 
-    if (fd > 0 && fd == console_fd)
+    if (fd > 0 && fd == tty_fd)
         return 0;
 
     _close = (int (*)(int fd)) dlsym (RTLD_NEXT, "close");
@@ -1950,7 +2000,7 @@ audit_open (void)
     if (g_key_file_get_boolean (config, "test-audit-config", "check-events", NULL))
         status_notify ("AUDIT OPEN");
 
-    return 1;
+    return dup (STDOUT_FILENO);
 }
 
 int
